@@ -1,11 +1,11 @@
 /* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * go-styled-object.c :  
+ * go-styled-object.c :
  *
  * Copyright (C) 2009 JJean Brefort (jean.brefort@normalesup.org)
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -23,6 +23,24 @@
 #include <goffice/goffice-config.h>
 #include <goffice/utils/go-styled-object.h>
 
+/**
+ * SECTION:go-styled-object
+ * @short_description: Objects with style
+ *
+ * The common GInterface for all objects owning a GOStyle.
+ **/
+
+
+/**
+ * GOStyledObjectClass :
+ * @set_style: sets the object style.
+ * @get_style: returns the object current style.
+ * @get_auto_style: returns the default style for the object.
+ * @style_changed: called when the style changed.
+ * @apply_theme: apply the current theme if any to the object style.
+ * @get_document: returns the #GODoc associated to the object if any.
+ **/
+
 GType
 go_styled_object_get_type (void)
 {
@@ -50,8 +68,43 @@ go_styled_object_get_type (void)
  * Sets a new style for @gso, and emits "style-changed" signal. This function
  * does not take ownership of @style.
  *
+ * The best way to change the style is to set the "style" property.
+ *
+ * This function will fail if the new style and the previous style are the same.
+ * In that case, the function will always return false:
+ * <informalexample>
+ *  <programlisting>
+ *      style = go_styled_object_get_style (gso);
+ *      style->line.width = 2;
+ *      size_changed = go_styled_object_set_style (gso, style);
+ *  </programlisting>
+ * </informalexample>
+ * In this sample, the call to go_styled_object_set_style() is just useless. You
+ * need to check yourself if you really change the size, call
+ * go_styled_object_style_changed() to trigger the "style-changed" event.
+ * So the following code is much better:
+ * <informalexample>
+ *  <programlisting>
+ *      style = go_styled_object_get_style (gso);
+ *      if (style->line.width != 2.) {
+ *	      style->line.width = 2;
+ *	      go_styled_object_style_changed (gso);
+ *	      size_changed = true;
+ *      } else
+ *	      size_changed = FALSE;
+ *  </programlisting>
+ * </informalexample>
+ * or even better:
+ * <informalexample>
+ *  <programlisting>
+ *      style = go_style_dup (go_styled_object_get_style (gso));
+ *      style->line.width = 2;
+ *      size_changed = go_styled_object_set_style (gso, style);
+ *      g_object_unref (style);
+ *  </programlisting>
+ * </informalexample>
  * return value: %TRUE if new style may lead to change of object size, which
- * happens when changing font size for example. 
+ * happens when changing font size for example.
  **/
 gboolean
 go_styled_object_set_style (GOStyledObject *gso, GOStyle *style)
@@ -67,7 +120,7 @@ go_styled_object_set_style (GOStyledObject *gso, GOStyle *style)
  * @gso: a #GOStyledObject
  *
  * Simply an accessor function that returns @gso->style, without referencing it.
- * 
+ *
  * return value: the styled object's #GOStyle
  **/
 GOStyle*
@@ -118,7 +171,7 @@ go_styled_object_style_changed (GOStyledObject *gso)
  * @gso: a #GOStyledObject
  * @style: a #GOStyle that will be themed
  *
- * Apply appropriate theme @style if meaningful, i.e. properties with 
+ * Apply appropriate theme @style if meaningful, i.e. properties with
  * auto flag set to %TRUE should be changed to default theme value.
  *
  **/
@@ -130,6 +183,14 @@ go_styled_object_apply_theme (GOStyledObject *gso, GOStyle *style)
 	if (klass->apply_theme)
 		klass->apply_theme (gso, style);
 }
+/**
+ * go_styled_object_get_document:
+ * @gso: a #GOStyledObject
+ *
+ * A #GODoc is necessary to store images. If no GODoc is associated with the
+ * object, image filling will not be supported.
+ * return value: the #GODoc associated with the object if any.
+ **/
 
 GODoc*
 go_styled_object_get_document (GOStyledObject *gso)
@@ -140,62 +201,36 @@ go_styled_object_get_document (GOStyledObject *gso)
 		klass->get_document (gso): NULL;
 }
 
+/**
+ * go_styled_object_set_cairo_line :
+ * @so: #GOStyledObject
+ * @cr: #cairo_t
+ *
+ * Prepares the cairo context @cr to draw a line according to the
+ * item style and canvas scale.
+ *
+ * Returns: %TRUE if the line is not invisible
+ **/
 gboolean
 go_styled_object_set_cairo_line (GOStyledObject const *so, cairo_t *cr)
 {
 	GOStyle const *style;
-	double width;
-	GOLineDashSequence *line_dash;
 
 	g_return_val_if_fail (GO_IS_STYLED_OBJECT (so), FALSE);
 	style = go_styled_object_get_style (GO_STYLED_OBJECT (so));
-	if (style->line.dash_type == GO_LINE_NONE)
-		return FALSE;
-	width = (style->line.width)? style->line.width: 1.;
-	cairo_set_line_width (cr, width);
-	cairo_set_line_cap (cr, style->line.cap);
-	cairo_set_line_join (cr, style->line.join);
-	cairo_set_miter_limit (cr, style->line.miter_limit);
-	line_dash = go_line_dash_get_sequence (style->line.dash_type, width);
-	if (style->line.cap == CAIRO_LINE_CAP_BUTT && style->line.dash_type != GO_LINE_SOLID) {
-		unsigned i;
-		for (i = 0; i < line_dash->n_dash; i++)
-			if (line_dash->dash[i] == 0.)
-				line_dash->dash[i] = width;
-	}
-	if (line_dash != NULL)
-		cairo_set_dash (cr,
-				line_dash->dash,
-				line_dash->n_dash,
-				line_dash->offset);
-	else
-		cairo_set_dash (cr, NULL, 0, 0);
-	switch (style->line.pattern) {
-	case GO_PATTERN_SOLID:
-		cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (style->line.color));
-		break;
-	case GO_PATTERN_FOREGROUND_SOLID:
-		cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (style->line.fore));
-		break;
-	default: {
-		GOPattern pat;
-		double scalex = 1., scaley = 1.;
-		cairo_pattern_t *cp;
-		cairo_matrix_t mat;
-		pat.fore = style->line.fore;
-		pat.back = style->line.color;
-		pat.pattern = style->line.pattern;
-		cp = go_pattern_create_cairo_pattern (&pat, cr);
-		cairo_user_to_device_distance (cr, &scalex, &scaley);
-		cairo_matrix_init_scale (&mat, scalex, scaley);
-		cairo_pattern_set_matrix (cp, &mat);
-		cairo_set_source (cr, cp);
-		break;
-	}
-	}
-	return TRUE;
+	return go_style_set_cairo_line (style, cr);
 }
 
+/**
+ * go_styled_object_set_cairo_fill :
+ * @so: #GOStyledObject
+ * @cr: #cairo_t
+ *
+ * Prepares the cairo context @cr to fill a shape according to the
+ * item style and canvas scale.
+ *
+ * Returns: %TRUE if the filling is not invisible
+ **/
 gboolean
 go_styled_object_set_cairo_fill (GOStyledObject const *so, cairo_t *cr)
 {
